#!/usr/bin/env python3

import os
import socket
import selectors
import threading
from argparse import ArgumentParser
import i3ipc
import dynmen
from shlex import split
import collections
import sys

SOCKET_FILE = '/tmp/i3_focus_last'
MAX_WIN_HISTORY = 15


class FocusWatcher:
    def __init__(self):
        self.i3 = i3ipc.Connection()
        self.i3.on('window::focus', self.on_window_focus)
        self.listening_socket = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        if os.path.exists(SOCKET_FILE):
            os.remove(SOCKET_FILE)
        self.listening_socket.bind(SOCKET_FILE)
        self.listening_socket.listen(1)
        self.window_list = []
        self.window_list_lock = threading.RLock()

    def on_window_focus(self, i3conn, event):
        with self.window_list_lock:
            window_id = event.container.id
            if window_id in self.window_list:
                self.window_list.remove(window_id)
            self.window_list.insert(0, window_id)
            if len(self.window_list) > MAX_WIN_HISTORY:
                del self.window_list[MAX_WIN_HISTORY:]

    def launch_i3(self):
        self.i3.main()

    def launch_server(self):
        selector = selectors.DefaultSelector()

        def accept(sock):
            conn, addr = sock.accept()
            selector.register(conn, selectors.EVENT_READ, read)

        def read(conn):
            data = conn.recv(1024)
            if data == b'switch':
                with self.window_list_lock:
                    tree = self.i3.get_tree()
                    windows = set(w.id for w in tree.leaves())
                    coll = []
                    for window_id, win_n in zip( self.window_list, range(len(self.window_list)) ):
                        if window_id not in windows:
                            self.window_list.remove(window_id)
                        else:
                            win = tree.find_by_id(window_id)
                            coll.append(("w{} d{:<4} | {}".format(win_n, win.workspace().name, win.name),
                                         window_id))
                    rofi = dynmen.Menu(split('rofi -dmenu -i -p \'> \'') + self.args)
                    print(coll)
                    try:
                        result = rofi(collections.OrderedDict(coll[1:]))
                        self.i3.command('[con_id=%s] focus' % result.value)
                    except dynmen.MenuError:
                        result = None
                        print("no selection")

            elif not data:
                selector.unregister(conn)
                conn.close()

        selector.register(self.listening_socket, selectors.EVENT_READ, accept)

        while True:
            for key, event in selector.select():
                callback = key.data
                callback(key.fileobj)

    def run(self, args):
        self.args = args
        t_i3 = threading.Thread(target=self.launch_i3)
        t_server = threading.Thread(target=self.launch_server)
        for t in (t_i3, t_server):
            t.start()


def server():
    args = sys.argv[1:]
    focus_watcher = FocusWatcher()
    focus_watcher.run(args)
    

def client():
    client_socket = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
    client_socket.connect(SOCKET_FILE)
    client_socket.send(b'switch')
    client_socket.close()


if __name__ == '__main__':
    server()



from dynmen import Menu
import i3ipc
from shlex import split
import collections
import sys

def main(rofi_args=sys.argv[1:]):
    command_line = "rofi -dmenu -i -markup-rows -p '> '"
    i3 = i3ipc.Connection()
    pink = "w{} d{:<4} |<span color='pink'> {}</span>"
    yellow = "w{} d{:<4} |<span color='yellow'> {}</span>"
    coll = collections.deque()
    t = i3.get_tree()
    f = t.find_focused()
    for w, n in zip(t.leaves(), range(len(t.leaves()))):
        if f.id == w.id:
            coll.appendleft((w.id, yellow.format(n + 1, w.workspace().name, w.name)))
        elif w.workspace().id == f.workspace().id:
            coll.appendleft((w.id, pink.format(n + 1, w.workspace().name, w.name)))
        else:
            coll.append((w.id, "w{} d{:<4} | {}".format(n + 1, w.workspace().name, w.name)))

    rofi = Menu(split(command_line) + rofi_args)
    result = rofi(collections.OrderedDict((v, k) for k, v in coll))
    f = t.find_by_id(result.value)
    f.command('focus')
    
if __name__ == '__main__':
    main()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="i3wins",
    version="0.0.4",
    author="Yoann Lamouroux",
    author_email="ylamouroux@ubuntu.com",
    description="Yet another i3 window switcher",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ylmrx/i3wins",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=['i3ipc', 'dynmen', 'traitlets'],
    entry_points={
        'console_scripts': [
            'i3wins=i3wins.i3wins:main',
            'i3lasts=i3wins.i3lasts:server',
            'i3lastc=i3wins.i3lasts:client'
        ]
    }
)

